package exercises.ex_o;

import java.util.Stack;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.ReentrantLock;

public class LockedWithConditionsStack {
    private final int capacity;
    private final Stack<Integer> stack = new Stack<>();
    private final ReentrantLock lock = new ReentrantLock();
    private final Condition fullCondition = lock.newCondition();
    private final Condition emptyCondition = lock.newCondition();

    public LockedWithConditionsStack(int capacity) {
        this.capacity = capacity;
    }

    public void pushToStack(int value) {
        try {
            lock.lock();
            while (stack.size() == capacity) {
                System.out.println("Stack is full, waiting until someone pops");
                fullCondition.await();
            }
            stack.push(value);
            emptyCondition.signalAll();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        } finally {
            lock.unlock();
        }
    }

    public int popFromStack() {
        int value;
        try {
            lock.lock();
            while (stack.isEmpty()) {
                System.out.println("Stack is empty, waiting until someone pushes");
                emptyCondition.await();
            }
            value = stack.pop();
            fullCondition.signalAll();
        } catch (InterruptedException e) {
            throw new RuntimeException(e);
        } finally {
            lock.unlock();
        }
        return value;
    }
}
